@@ -8,6 +8,7 @@
 """
 import ctypes
 import ctypes.util
+import itertools
 import os
 import os.path
 import sys
@@ -84,7 +85,10 @@ def load_library():
 
     """
     tried_paths = []
-    for suffix in '', '-Q16', '-Q8', '-6.Q16':
+    versions = ('', '-Q16', '-Q8', '-6.Q16')
+    options = ('', 'HDRI')
+    combinations = itertools.product(versions, options)
+    for suffix in (version + option for version, option in combinations):
         libwand_path, libmagick_path = find_library(suffix)
         if libwand_path is None or libmagick_path is None:
             continue
@@ -282,6 +286,11 @@ try:
 
     library.MagickSetImageUnits.argtypes = [ctypes.c_void_p, ctypes.c_int]
 
+    library.MagickGetImageColorspace.argtypes = [ctypes.c_void_p]
+    library.MagickGetImageColorspace.restype = ctypes.c_int
+
+    library.MagickSetImageColorspace.argtypes = [ctypes.c_void_p, ctypes.c_int]
+
     library.MagickGetImageDepth.argtypes = [ctypes.c_void_p]
     library.MagickGetImageDepth.restype = ctypes.c_size_t
 
@@ -304,6 +313,9 @@ try:
 
     library.MagickResetImagePage.argtypes = [ctypes.c_void_p, ctypes.c_char_p]
 
+    library.MagickSampleImage.argtypes = [ctypes.c_void_p, ctypes.c_size_t,
+                                          ctypes.c_size_t]
+                                          
     library.MagickResizeImage.argtypes = [ctypes.c_void_p, ctypes.c_size_t,
                                           ctypes.c_size_t, ctypes.c_int,
                                           ctypes.c_double]
@@ -463,6 +475,12 @@ try:
                                                 ctypes.c_double,
                                                 ctypes.c_double]
 
+    library.MagickUnsharpMaskImage.argtypes = [ctypes.c_void_p,
+                                               ctypes.c_double,
+                                               ctypes.c_double,
+                                               ctypes.c_double,
+                                               ctypes.c_double]
+
     library.MagickGetNumberImages.argtypes = [ctypes.c_void_p]
     library.MagickGetNumberImages.restype = ctypes.c_size_t
 
